#include "stdafx.h"
#include "Retezec.h"
#include <conio.h>
#include <stdio.h>

Retezec::Retezec(const Retezec & ret)
{
	// inicializujeme tento pomoci existujiciho retezce, pouze se pripojime
	// na jeho RetRep a inkrementujeme pocet
	rr = ret.rr;
	rr->pocet++;
}

Retezec::Retezec(const char * str)
{
	// vytvorime novou strukturu a zapamatujeme si ukazatel na ni
	rr = new Retezec::RetRep;
	// je zatim jeden (ten nas) ukazatel, co na ni ukazuje
	rr->pocet = 1;
	// a naplnime jeji druhou polozku daty
	rr->str = new char[strlen(str)+1];
	strcpy(rr->str, str);
}

Retezec::~Retezec()
{
	if (--rr->pocet == 0) // snizime pocet
	{
		// je treba odalokovat, pocet ukazatelu klesnul na nulu
		delete[] rr->str;
		delete rr;
	}
}

int Retezec::Delka() const
{
	// jednoduse vracime delku
	// pretypovani je jen detail, strlen() vraci size_t, coz je typedef na neco,
	// asi na long, tak to hlasi bez pretypovani warning..
	return (int)strlen(rr->str);
}

Retezec Retezec::operator+(const Retezec & ret) const
{
	// neimplementujeme, pouzijeme druhy operator a podstrcime mu parametr, ktery chce
	return operator+(ret.rr->str);
}

Retezec Retezec::operator+(const char * str) const
{
	// naalokujeme si policko dostatecne dlouhe pro spojeny retezec
	char * pomocne = new char[this->Delka() + strlen(str) + 1];
	// retezce tam nakopirujeme
	strcpy(pomocne, this->rr->str);
	strcat(pomocne, str);
	// vytvorime retezec s novym obsahem
	Retezec vysledek(pomocne);
	// pomocne pole nezapomeneme odalokovat
	delete[] pomocne;
	// a vysledek vracime
	return vysledek;
}


bool Retezec::operator==(const Retezec & ret) const
{
	// opet volame druhou verzi
	return operator==(ret.rr->str);
}

bool Retezec::operator==(const char * str) const
{
	// jednoduse porovname retezce pomoci strcmp(), pokud vrati 0, jsou shodne
	// a vracime true
	return strcmp(rr->str, str) == 0;
}

bool Retezec::operator!=(const Retezec & ret) const
{
	// != je opak ==
	return !operator==(ret);
}

bool Retezec::operator!=(const char * str) const
{
	return !operator==(str);
}

char & Retezec::operator[](int index)
{
	// kontrola hodnoty indexu je potreba, nevime co dostaneme
	if (index < 0 || index > Delka() - 1)
		Chyba("index mimo retezec");

	// vratime prislusny znak z retezce, ten se vraci referenci, takze pujde i menit
	return rr->str[index];
}

const char & Retezec::operator[](int index) const
{
	if (index < 0 || index > Delka() - 1)
		Chyba("index mimo retezec");

	return rr->str[index];
}

Retezec::operator const char *() const
{
	// vratime ukazatel na nase vnitrni data, protoze je ale const, tak to nevadi,
	// nikdo nam je nemuze zmenit
	return rr->str;
}

Retezec & Retezec::operator=(const Retezec & ret)
{
	// kontrola, zda nahodou neprirazujeme stejny retezec do stejneho, to by delalo
	// neplechu. V tom pripade se nema delat nic, tak hned skoncime
	if (this->rr == ret.rr)
		return *this;

	// pokud jsme posledni, kdo vlastni nase data (my uz je nepotrebujeme), tak
	// je uvolnime
	if (rr->pocet == 1)
	{
		delete[] rr->str;
		delete rr;
	}
	else // kdyz nejsme posledni, tak akorat snizime pocet, my se odpojujeme
		rr->pocet--;

	// a ukazeme si na data praveho operandu a zvysime jim pocet
	rr = ret.rr;
	rr->pocet++;

	return *this;
}

Retezec & Retezec::operator=(const char * str)
{
	if (rr->pocet > 1) // na nas retezec vede vice odkazu, musime se od nej odpojit
	{                  // a vytvorit si jiny
		rr->pocet--;
		rr = new Retezec::RetRep;
		rr->pocet = 1; // posleze priradime retezec a budeme sami kdo si na nej ukazuje
	}
	else // jsme posledni, odalokujeme ho, jednicka v rr->pocet zustava
		delete[] rr->str;

	// naalokujeme novy retezec a zkopirujeme do nej data
	rr->str = new char[strlen(str) + 1];
	strcpy(rr->str, str);

	return *this;
}


void Chyba(const char * str)
{
	printf("Chyba: ");
	puts(str);
	getch();
	exit(-1);
}

