#ifndef _RETEZEC_H_
#define _RETEZEC_H_

//
// Rozdily oproti verzi na hodine:
// 1) Zapomneli jsme na destruktor, zde je pridany
// 2) Konstruktor Retezec::Retezec(const Retezec & ret) nemusi nic alokovat,
//    zde je lepsi verze nez byla na hodine
// 3) V metode Retezec Retezec::operator+(const char * str) const
//    jsme zapomneli odalokovat pomocne pole znaku, opraveno
// 4) Pridal jsem jeste operator= ktery bere jako parametr (char *), ten se totiz
//    od verze co bere Retezec & dost lisi
// 5) Pridan jeste operator konverze na (const char *)
//

// Takto by mohla vypadat trida pro retezce, ktera je rozumne pouzitelna a
// pretezuje nektere operatory tak, ze se chovaji podobne jako u zakladnich typu
// jako int, double atd..
class Retezec
{
	// Retezec si neukazuje primo na char *, ale zprostredkovane pres tuto strukturu,
	// ktera jeste umoznuje si zapamatovat, z kolika retezcovych objektu se na ni
	// ukazuje. To oddaluje kopirovani retezcu az na dobu, kdy je to nezbytne nutne
	// a pokud to nebude nutne, usetri se cas i pamet.
	struct RetRep
	{
		char * str; // opravdova data - normalni pole znaku ukoncene nulou
		int pocet; // pocet retezcovych objektu ukazujicich si na tato data
	};
	
	// retezcova trida ma ukazatel na strukturu RetRep alias RetezcovaReprezentace
	RetRep * rr;

public:
	// kopirovaci konstruktor
	Retezec(const Retezec & ret);
	// konstruktor z obycejneho pole znaku, pokud se nespecifikuje nic, vytvori se
	// prazdny retezec
	Retezec(const char * str = "");
	// destruktor snizi pocet ukazatelu na strukturu rr, pripadne ji odalokuje,
	// pokud tento objekt ma posledni ukazatel na ni
	~Retezec();

	// vrati delku retezce
	int Delka() const;

	// umozni korektne priradit jeden retezec do druheho
	Retezec & operator=(const Retezec & ret);
	Retezec & operator=(const char * str);

	// spoji dva retezce a vrati novy objekt reprezentujici novy spojeny retezec
	Retezec operator+(const Retezec & ret) const;
	// jeste jedno pretizeni pro rychlejsi spojovani s cckovymi retezci
	Retezec operator+(const char * str) const;

	// porovna lexikograficky dva retezce
	bool operator==(const Retezec & ret) const;
	// a zase pretizeni pro rychlejsi porovnani s cckovym retezcem
	// vsimete si, ze to neumoznuje porovnavat ("ahoj" == Retezec("nazdar"))
	bool operator==(const char * str) const;

	// a dalsi porovnavaci operatory
	bool operator!=(const Retezec & ret) const;
	bool operator!=(const char * str) const;
	bool operator<(const Retezec & ret) const;
	bool operator<(const char * str) const;
	bool operator>(const Retezec & ret) const;
	bool operator>(const char * str) const;
	bool operator<=(const Retezec & ret) const;
	bool operator<=(const char * str) const;
	bool operator>=(const Retezec & ret) const;
	bool operator>=(const char * str) const;

	// pripoji na konec aktualniho retezce znak
	Retezec & operator+=(char ch);
	// pripoji na konec aktualniho retezce jiny retezec
	Retezec & operator+=(const Retezec & ret);
	Retezec & operator+=(const char * str);

	// metody, ktere delaji totez co operatory +=
	Retezec & Pripoj(char ch);
	Retezec & Pripoj(const Retezec & ret);
	Retezec & Pripoj(const char * str);

	// verze operatoru [], ktera se vola pri nekonstantnim objektu
	char & operator[](int index);
	// a verze pro konstantni objekt, ktera dovoli alespon cist
	const char & operator[](int index) const;

	// aby sel retezec pouzivat vsude tam, kde se ocekava (char *)
	operator const char *() const;

	// tento operator volani funkce by se mohl pouzit pro ziskavani podretezce
	// tj. bude delat totez, co metoda Podretezec() - nejlepe kdyby ji volal.
	Retezec operator()(int index, int pocet) const;
	Retezec Podretezec(int index, int pocet) const;

	// Vlozi retezec na dany index do naseho retezce
	Retezec & Vloz(int index, const Retezec & ret);
	// totez pro rychlejsi praci s  cckovymi retezci
	Retezec & Vloz(int index, const char * str);
	// odstrani dany pocet znaku z daneho indexu z retezce
	Retezec & Odstran(int index, int pocet);

	// najde v retezci dany znak, pripadne zacne na specifikovanem indexu.
	// vrati index, kde se ten znak nachazi, pokud v retzci neni, vrati -1
	int Hledej(char ch, int start = 0);
	// totez, ale hleda vyskyt podretezce
	int Hledej(const Retezec & ret, int start = 0);

	// vrati novy retezec, ktery obsahuje prevedena vsechna velka pismena na mala.
	// Ostatni znaky nemeni
	Retezec NaMala() const;
	// totez, ale prevadi mala pismena na velka
	Retezec NaVelka() const;

	// vrati retezec, ktery je identicky s timto, ale neobsahuje mezery a tabulatory
	// na zacatku a na konci, pokud tam jsou. Kdyz nejsou, jen vytvori novy retezec a
	// ten vrati.
	Retezec OdstranBileZnaky() const;
};

// funkce, ktera se zavola, kdyz nastane nejaka chyba
void Chyba(const char * str);

#endif










